//
//  CurrentAgeViewController.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 7/30/17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import UIKit
import SpriteKit

class CurrentAgeViewController: UIViewController, UIPickerViewDelegate, UIPickerViewDataSource {
    
    @IBOutlet weak var spriteKitView: SKView!
    @IBOutlet weak var picker: UIPickerView!
    @IBOutlet weak var label: UILabel!
    @IBOutlet weak var titleLabel: UILabel!
    @IBOutlet weak var nextButton: UIButton!
    
    
    var agesData: [Int] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Picker delegates
        picker.dataSource = self
        picker.delegate = self
        
        // Create the ages options for the picker
        for i in 7 ... 101 {
            agesData.append(i)
        }
        
        // Format the label
        let attributedString = NSMutableAttributedString(string: label.text!)
        attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
        attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-LightItalic", size: 16.0) as Any, range: NSRange(location: 0, length: attributedString.length))
        label.attributedText = attributedString
        
        // Select the default age or the saved age
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        var currentAgeIndex = 0
        for i in 0 ... agesData.count-1 {
            if agesData[i] == appDelegate.startAge {
                currentAgeIndex = i
            }
        }
        picker.selectRow(currentAgeIndex, inComponent: 0, animated: false)
        
        //view.backgroundColor = UIColor.clear
        
//        spriteKitView.showsFPS = true
//        spriteKitView.showsNodeCount = true
//        spriteKitView.showsPhysics = true
    }
    

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    
    // MARK: - Delegates and data sources
    // MARK: Data Sources

    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return agesData.count
    }

    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return String(agesData[row])
    }

    func pickerView(_ pickerView: UIPickerView, viewForRow row: Int, forComponent component: Int, reusing view: UIView?) -> UIView {
        var pickerLabel = view as! UILabel!
        if view == nil {  //if no label there yet
            pickerLabel = UILabel()
        }
        let titleData = String(agesData[row])
        
        let scene:FallingBonesScene = spriteKitView.scene as! FallingBonesScene
        let current = pickerView.selectedRow(inComponent: component)
        scene.onPickerChange(value: agesData[current])
        
        var fontSize:CGFloat = 51.0
        // 1. request an UITraitCollection instance
        let deviceIdiom = UIScreen.main.traitCollection.userInterfaceIdiom
        // 2. check the idiom
        switch (deviceIdiom) {
        case .pad:
            fontSize = 72.0
        case .phone:
            fontSize = 52.0
        case .tv:
            fontSize = 96.0
        default:
            fontSize = 52.0
        }
        let pickerFont = UIFont(name:"Graphik-LightItalic", size: fontSize)
        let myTitle = NSAttributedString(string: titleData, attributes: [NSAttributedStringKey.font:pickerFont, NSAttributedStringKey.foregroundColor:UIColor.white])
        
        pickerLabel!.attributedText = myTitle
        pickerLabel!.textAlignment = .center
        
        return pickerLabel!
    }
    
    func pickerView(_ pickerView: UIPickerView, rowHeightForComponent component: Int) -> CGFloat {
        // 1. request an UITraitCollection instance
        let deviceIdiom = UIScreen.main.traitCollection.userInterfaceIdiom
        // 2. check the idiom
        switch (deviceIdiom) {
        case .pad:
            return 80.0
        default:
            return 64.0
        }
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        appDelegate.startAge = agesData[row]
        
        let scene:FallingBonesScene = spriteKitView.scene as! FallingBonesScene
        scene.onPickerChange(value: appDelegate.startAge)
    }

    override func viewWillDisappear(_ animated: Bool) {
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        let current = picker.selectedRow(inComponent: 0)
        appDelegate.startAge = agesData[current]

    }
    
    func prepViews() {
        titleLabel.alpha = 0
        picker.alpha = 0
        label.alpha = 0
        nextButton.alpha = 0
        spriteKitView.alpha = 1
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        prepViews()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        UIView.animate(withDuration: 3.0, delay: 0.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.spriteKitView.alpha = 0.33
        }, completion: nil)
        
        UIView.animate(withDuration: 3.0, delay: 0.5, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.titleLabel.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 2.0, delay: 1.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.picker.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 2.0, delay: 1.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.label.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 1.0, delay: 1.5, options:[.curveEaseOut,.allowUserInteraction], animations: {
            self.nextButton.alpha = 1.0
        }, completion: nil)
        

        
    }
    
//    override var prefersStatusBarHidden: Bool {
//        return true
//    }
    
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
