//
//  EndAgeViewController.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 7/30/17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import UIKit
import Lottie

class EndAgeViewController: UIViewController {
    
    @IBOutlet weak var slider: UISlider!
    @IBOutlet weak var sliderLabel: UILabel!
    @IBOutlet weak var sliderValue: UILabel!
    @IBOutlet weak var skullContainer: UIView!
    @IBOutlet weak var titleView: UILabel!
    @IBOutlet weak var nextButton: UIButton!
    
    
    var endAgeOptions: [Int] = []
    let blinkView = LOTAnimationView(name: "blinking_skull_blink")
    let jawView = LOTAnimationView(name: "blinking_skull_jaw")
    
    override func viewDidLoad() {
        super.viewDidLoad()

        // Fix up the label with styling that you can't do in IB
        let attributedString = NSMutableAttributedString(string: sliderLabel.text!)
        attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
        attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-LightItalic", size: 16.0) as Any, range: NSRange(location: 0, length: attributedString.length))
        sliderLabel.attributedText = attributedString
        
        // Create the ages options
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        var startAge:Int = appDelegate.startAge
        let maxAge:Int = appDelegate.maxAge
        
        // Give 'em at least a year to live for God's sake!
        startAge = startAge + 1
        for i in startAge ... maxAge {
            endAgeOptions.append(i)
        }

        // Set the slider in the right position based on saved endAge value
        var percent:Float = 0.5
        if (appDelegate.endAge > startAge) {
            percent = Float(appDelegate.endAge-startAge) / Float(maxAge-startAge)
        }
        //print(percent, appDelegate.endAge, startAge, maxAge)
        slider.setValue(percent, animated: false)
        updateSliderValueLabel(string:appDelegate.endAge.description)
        
        jawView.contentMode = .scaleAspectFit
        jawView.loopAnimation = true
        jawView.animationSpeed = 1.0
        jawView.animationProgress = CGFloat(percent)
        skullContainer.addSubview(jawView)
        
        blinkView.contentMode = .scaleAspectFit
        blinkView.loopAnimation = true
        blinkView.play()
        blinkView.animationSpeed = 1.0
        skullContainer.addSubview(blinkView)
        
        
//        let min = CGFloat(-30)
//        let max = CGFloat(30)
        
//        let minRot = Measurement(value: Double(min), unit: UnitAngle.degrees)
//            .converted(to: .radians).value
//        let maxRot = Measurement(value: Double(max), unit: UnitAngle.degrees)
//            .converted(to: .radians).value
        
//        let xMotion = UIInterpolatingMotionEffect(keyPath: "layer.transform.translation.x", type: .tiltAlongHorizontalAxis)
//        xMotion.minimumRelativeValue = min
//        xMotion.maximumRelativeValue = max
//
//        let yMotion = UIInterpolatingMotionEffect(keyPath: "layer.transform.translation.y", type: .tiltAlongVerticalAxis)
//        yMotion.minimumRelativeValue = min
//        yMotion.maximumRelativeValue = max
//
//        let motionEffectGroup = UIMotionEffectGroup()
//        motionEffectGroup.motionEffects = [xMotion,yMotion]
//
//        skullContainer.addMotionEffect(motionEffectGroup)
        
        
    }
    
    override func viewWillLayoutSubviews() {
        blinkView.frame = CGRect(x: 0, y: 0, width: skullContainer.frame.width, height: skullContainer.frame.height)
        jawView.frame = CGRect(x: 0, y: 0, width: skullContainer.frame.width, height: skullContainer.frame.height)
        //print(blinkView.frame, skullContainer.frame)

    }
    
    override func viewWillDisappear(_ animated: Bool) {
        // Update the icon badge when dismissing this view
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        UIApplication.shared.applicationIconBadgeNumber = appDelegate.numberOfWeeksLeft
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    @IBAction func onSliderChange(_ sender: Any) {
        
        let valIndex = Int(round(slider.value * Float(endAgeOptions.count - 1)))
        let val = endAgeOptions[valIndex]
        let valStr = String(val)
        
        // Save settings
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        appDelegate.endAge = val
        
        // Calculate number of weeks left
        let years = appDelegate.endAge - appDelegate.startAge
        let weeks = 52 * years
        appDelegate.numberOfWeeksLeft = weeks
        
        updateSliderValueLabel(string:valStr)
        
        jawView.animationProgress = CGFloat(slider.value)
    }
    
    func updateSliderValueLabel(string:String) {
        let attributedString = NSMutableAttributedString(string: string)
        attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
        sliderValue.attributedText = attributedString
    }
    
    func prepViews() {
        titleView.alpha = 0
        slider.alpha = 0
        sliderLabel.alpha = 0
        sliderValue.alpha = 0
        nextButton.alpha = 0
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        prepViews()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        UIView.animate(withDuration: 3.0, delay: 0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.titleView.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 2.0, delay: 1.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.sliderLabel.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 2.0, delay: 1.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.slider.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 2.0, delay: 1.0, options:[.curveEaseOut, .allowUserInteraction], animations: {
            self.sliderValue.alpha = 1.0
        }, completion: nil)
        
        UIView.animate(withDuration: 1.0, delay: 1.5, options:[.curveEaseOut,.allowUserInteraction], animations: {
            self.nextButton.alpha = 1.0
        }, completion: nil)
        
       
    }
    
//    override var prefersStatusBarHidden: Bool {
//        return true
//    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
