//
//  ResultsViewController.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 7/30/17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import QuartzCore
import UIKit
import SpriteKit

protocol ProgressLayerProtocol {
    func progressUpdated(progress: CGFloat)
}

class ProgressLayer : CALayer {
    @NSManaged
    var progress: CGFloat
    var progressDelegate: ProgressLayerProtocol? = nil
    
    override class func needsDisplay(forKey key: String) -> Bool {
        if (key == "progress") {
            return true
        }
        return super.needsDisplay(forKey: key)
    }
    
    override func draw(in ctx: CGContext) {
        super.draw(in: ctx)
        if let del = self.delegate {
            let c = del as? ProgressLayerProtocol
            c?.progressUpdated(progress: progress)
        }
    }
}

class ResultsViewController: UIViewController, CAAnimationDelegate, CALayerDelegate, ProgressLayerProtocol {
    
    
    @IBOutlet weak var weekLeftTitle: UILabel!
    
    @IBOutlet weak var weeksLeftLabel: UILabel!
    @IBOutlet weak var weeksLeftValue: UILabel!
    
    @IBOutlet weak var weaksLivedLabel: UILabel!
    @IBOutlet weak var weaksLivedValue: UILabel!
    
    @IBOutlet weak var startAgeLabel: UILabel!
    @IBOutlet weak var endAgeLabel: UILabel!
    
    @IBOutlet weak var yearsLivedArc: ArcView!
    @IBOutlet weak var yearsLeftArc: ArcView!
    
    @IBOutlet weak var startOverButton: UIButton!
    @IBOutlet weak var infoButton: UIButton!
    
    @IBOutlet weak var rotatingArc1: RotatingArcView!
    @IBOutlet weak var rotatingArc2: RotatingArcView!
    @IBOutlet weak var rotatingArc3: RotatingArcView!
    
    @IBOutlet weak var StartfieldView: SKView!
    
    var startAge: CGFloat = 7
    var endAge: CGFloat = 107
    var numberOfWeeksLeft: Int = 4000
    var percentOfLifeLived:CGFloat = 0
    var ringStart:CGFloat = 0
    var ringEnd:CGFloat = 0
    var startEndPercent:CGFloat = 0

    // Animation related
    var progressLayer2: ProgressLayer!
    var isInPhase1: Bool = false
    var isInPhase2: Bool = false
    var animPhase1: CASpringAnimation!
    var animPhase2: CASpringAnimation!
    var ring1Animation: CASpringAnimation!
    var ring2Animation: CASpringAnimation!
    
    //var arc1FadeIn: UIViewPropertyAnimator!
    //var arc1FadeOut: UIViewPropertyAnimator!
    
    var arc1FadeIn  = CABasicAnimation(keyPath:"opacity")
    var arc1FadeOut = CABasicAnimation(keyPath:"opacity")
    var arc2FadeIn  = CABasicAnimation(keyPath:"opacity")
    var arc2FadeOut = CABasicAnimation(keyPath:"opacity")
    var arc3FadeIn  = CABasicAnimation(keyPath:"opacity")
    var arc3FadeOut = CABasicAnimation(keyPath:"opacity")
    
    var bigFontSize:CGFloat = 52.0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let appDelegate = UIApplication.shared.delegate as! AppDelegate
        startAge = CGFloat(appDelegate.startAge)
        endAge = CGFloat(appDelegate.endAge)
        numberOfWeeksLeft = appDelegate.numberOfWeeksLeft

        // Fix up the label with styling that you can't do in IB
        var attributedString = NSMutableAttributedString(string: weeksLeftLabel.text!)
        attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
        weeksLeftLabel.attributedText = attributedString
        
        attributedString = NSMutableAttributedString(string: weaksLivedLabel.text!)
        attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-LightItalic", size: 16.0), range: NSRange(location: 0, length: attributedString.length))
        attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
        weaksLivedLabel.attributedText = attributedString

        // Configure the arcs
        percentOfLifeLived = startAge / endAge
        ringStart = 0.2
        ringEnd = 0.8
        startEndPercent = ((ringEnd - ringStart) * percentOfLifeLived) + 0.2
        
        // Give the foreground arc a shadow
//        yearsLivedArc.arc.shadowColor = UIColor.init(red: 0, green: 0, blue: 0, alpha: 0.2).cgColor
//        yearsLivedArc.arc.shadowRadius = 5.0
//        yearsLivedArc.arc.shadowOpacity = 1.0
//        yearsLivedArc.arc.shadowOffset = CGSize.zero
        
        // Add a hidden progress layer
        // Used only as a CAAnimation callback mechanism
        progressLayer2 = ProgressLayer()
        progressLayer2.frame = CGRect(x:0, y:0, width:1, height:1)
        progressLayer2.backgroundColor = UIColor.black.cgColor
        view.layer.addSublayer(progressLayer2)
        progressLayer2.delegate = self
        
        // Define the animations
        let g1Damping:CGFloat = 10.0
        let g1Stiffness:CGFloat = 10.0
        let g1Mass:CGFloat = 0.5
        
        let g2Damping:CGFloat = 60
        let g2Stiffness:CGFloat = 60
        let g2Mass:CGFloat = 20
        
        animPhase1 = CASpringAnimation(keyPath:"progress")
        animPhase1.fromValue = 0
        animPhase1.toValue = 1
        animPhase1.damping = g1Damping
        animPhase1.stiffness = g1Stiffness
        animPhase1.mass = g1Mass
        animPhase1.duration = animPhase1.settlingDuration
        animPhase1.fillMode = kCAFillModeForwards
        animPhase1.isRemovedOnCompletion = true
        animPhase1.delegate = self
        
        ring1Animation = CASpringAnimation(keyPath:"strokeEnd")
        ring1Animation.fromValue = CGFloat(ringStart)
        ring1Animation.toValue = startEndPercent
        ring1Animation.damping = g1Damping
        ring1Animation.stiffness = g1Stiffness
        ring1Animation.mass = g1Mass
        ring1Animation.duration = ring1Animation.settlingDuration
        ring1Animation.fillMode = kCAFillModeForwards
        ring1Animation.isRemovedOnCompletion = false
        ring1Animation.delegate = self
        
        animPhase2 = CASpringAnimation(keyPath:"progress")
        animPhase2.fromValue = 0
        animPhase2.toValue = 1
        animPhase2.damping = g2Damping
        animPhase2.stiffness = g2Stiffness
        animPhase2.mass = g2Mass
        animPhase2.duration = animPhase2.settlingDuration
        animPhase2.fillMode = kCAFillModeForwards
        animPhase2.isRemovedOnCompletion = true
        animPhase2.delegate = self
        
        ring2Animation = CASpringAnimation(keyPath:"strokeEnd")
        ring2Animation.fromValue = startEndPercent
        ring2Animation.toValue = ringEnd
        ring2Animation.damping = g2Damping
        ring2Animation.stiffness = g2Stiffness
        ring2Animation.mass = g2Mass
        ring2Animation.duration = ring2Animation.settlingDuration
        ring2Animation.fillMode = kCAFillModeForwards
        ring2Animation.isRemovedOnCompletion = false
        ring2Animation.delegate = self
        
        // Arc 1 Fade In
        arc1FadeIn.duration = 5.0
        arc1FadeIn.fromValue = 0.0
        arc1FadeIn.toValue = 1.0
        arc1FadeIn.fillMode = kCAFillModeForwards
        arc1FadeIn.isRemovedOnCompletion = false
        arc1FadeIn.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc1FadeIn.delegate = self
        
        // Arc 1 Fade Out
        arc1FadeOut.duration = 5.0
        arc1FadeOut.fromValue = 1.0
        arc1FadeOut.toValue = 0.0
        arc1FadeOut.fillMode = kCAFillModeForwards
        arc1FadeOut.isRemovedOnCompletion = false
        arc1FadeOut.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc1FadeOut.delegate = self

        // Arc 2 Fade In
        arc2FadeIn.duration = 5.0
        arc2FadeIn.fromValue = 0.0
        arc2FadeIn.toValue = 1.0
        arc2FadeIn.fillMode = kCAFillModeForwards
        arc2FadeIn.isRemovedOnCompletion = false
        arc2FadeIn.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc2FadeIn.delegate = self
        
        // Arc 2 Fade Out
        arc2FadeOut.duration = 5.0
        arc2FadeOut.fromValue = 1.0
        arc2FadeOut.toValue = 0.0
        arc2FadeOut.fillMode = kCAFillModeForwards
        arc2FadeOut.isRemovedOnCompletion = false
        arc2FadeOut.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc2FadeOut.delegate = self
        
        // Arc 3 Fade In
        arc3FadeIn.duration = 5.0
        arc3FadeIn.fromValue = 0.0
        arc3FadeIn.toValue = 1.0
        arc3FadeIn.fillMode = kCAFillModeForwards
        arc3FadeIn.isRemovedOnCompletion = false
        arc3FadeIn.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc3FadeIn.delegate = self
        
        // Arc 3 Fade Out
        arc3FadeOut.duration = 5.0
        arc3FadeOut.fromValue = 1.0
        arc3FadeOut.toValue = 0.0
        arc3FadeOut.fillMode = kCAFillModeForwards
        arc3FadeOut.isRemovedOnCompletion = false
        arc3FadeOut.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        arc3FadeOut.delegate = self

//        arc1FadeIn = UIViewPropertyAnimator(duration: 5, curve: .easeIn) {
//            self.rotatingArc1.alpha = 1
//        }
//        arc1FadeIn.addCompletion {
//            _ in
//            print("arc1FadeIn complete")
//            self.rotatingArc1.alpha = 1
//            self.fadeOutArc1()
//        }
//        arc1FadeOut = UIViewPropertyAnimator(duration: 5, curve: .easeIn) {
//            self.rotatingArc1.alpha = 0
//        }
//        arc1FadeOut.addCompletion {
//            _ in
//            print("arc1FadeOut complete")
//            self.rotatingArc1.alpha = 0
//            self.fadeInArc1()
//        }
        
        // prepViews()
        
        
    }
    
    override func viewDidLayoutSubviews() {
        bigFontSize = weaksLivedValue.font.pointSize
        //print(bigFontSize)
    }
    

    
    
    func prepViews() {
        weekLeftTitle.alpha = 0
        startAgeLabel.alpha = 0
        weaksLivedLabel.alpha = 0
        yearsLivedArc.alpha = 0
        weaksLivedValue.alpha = 0
        yearsLivedArc.strokeStart = ringStart
        yearsLivedArc.strokeEnd = startEndPercent
        
        yearsLeftArc.strokeStart = startEndPercent
        yearsLeftArc.strokeEnd = ringEnd
        yearsLeftArc.alpha = 0
        weeksLeftLabel.alpha = 0
        weeksLeftValue.alpha = 0
        endAgeLabel.alpha = 0
        
        startOverButton.alpha = 0
        infoButton.alpha = 0
        
        rotatingArc1.alpha = 0
        rotatingArc2.alpha = 0
        rotatingArc3.alpha = 0
        
        //StartfieldView.alpha = 0.0
        
       //fadeInArc1()
//        // Then fade rotating arc 1
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 5.0, delay: 1.0, options: [.curveEaseOut], animations: {
//            self.rotatingArc1.alpha = 1
//        }, completion: nil)
//
//        // Then fade rotating arc 2
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 5.0, delay: 5.6, options: [.curveEaseOut], animations: {
//            self.rotatingArc2.alpha = 1
//        }, completion: nil)
//
//        // Then fade rotating arc 3
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 5.0, delay: 10.6, options: [.curveEaseOut], animations: {
//            self.rotatingArc3.alpha = 1
//        }, completion: nil)
        
        
        
    }
    
//    func fadeInArc1() {
//        //arc1FadeIn.stopAnimation(false)
//        //arc1FadeOut.stopAnimation(false)
//        //arc1FadeIn.startAnimation(afterDelay: Double(arc4random_uniform(10)))
//
//        let duration = 5.0
//        let delay = duration + Double(arc4random_uniform(10))
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: duration, delay: delay, options: [.curveEaseOut], animations: {
//            self.rotatingArc1.alpha = 1
//        }, completion: {
//            position in
//            if position == .end {
//                print("fadeInArc1 complete")
//                self.fadeOutArc1()
//            }
//        })
//    }
//
//    func fadeOutArc1() {
//        //arc1FadeIn.stopAnimation(false)
//        //arc1FadeOut.stopAnimation(false)
//        //arc1FadeOut.startAnimation(afterDelay: Double(arc4random_uniform(10)))
//
//        let duration = 5.0
//        let delay = duration + Double(arc4random_uniform(10))
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: duration, delay: delay, options: [.curveEaseOut], animations: {
//            self.rotatingArc1.alpha = 0
//        }, completion: {
//            position in
//            if position == .end {
//                print("fadeOutArc1 complete")
//                self.fadeInArc1()
//            }
//        })
//    }
    
    
    
    func restartAnimation() {
        
        isInPhase1 = true
        isInPhase2 = false
        
        yearsLivedArc.arc.add(ring1Animation, forKey: "strokeEnd")
        
        progressLayer2.add(animPhase1, forKey: "progress")
        
//        // Fade in StartfieldView
//        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 5.0, delay: 0.0, options: [.curveEaseInOut], animations: {
//            self.StartfieldView.alpha = 1
//        }, completion:{_ in
//
//        })
        
        // Show the start over button
        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 3.0, delay: 2.0, options: [.curveEaseOut, .allowUserInteraction], animations: {
            self.startOverButton.alpha = 1
        }, completion: nil)
        
        // Show the info button
        UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 3.0, delay: 0.1, options: [.curveEaseOut, .allowUserInteraction], animations: {
            self.infoButton.alpha = 1
        }, completion: nil)
        
        arc1FadeIn.beginTime = CACurrentMediaTime() + 10.0
        rotatingArc1.layer.add(arc1FadeIn, forKey: "fadeIn")
        
        arc2FadeIn.beginTime = CACurrentMediaTime() + 17.0
        rotatingArc2.layer.add(arc2FadeIn, forKey: "fadeIn")
        
        //arc3FadeIn.beginTime = CACurrentMediaTime() + 25.0
        //rotatingArc3.layer.add(arc3FadeIn, forKey: "fadeIn")
        
    }
    
    func progressUpdated(progress: CGFloat) {
        
        var growingAge = 0
        var attributedString:NSMutableAttributedString
        
        if isInPhase1 {
            
            yearsLivedArc.alpha = progress
            startAgeLabel.alpha = progress
            weaksLivedLabel.alpha = progress
            
            // Weeks lived incrementing label
            var weekCount = Int(round(progress * CGFloat(52*startAge)))
            
            if (progress == 0.0) {
                weekCount = Int(CGFloat(52*startAge))
                yearsLivedArc.alpha = 1.0
                startAgeLabel.alpha = 1.0
                weaksLivedValue.alpha = 1.0
            }
            
            let numberFormatter = NumberFormatter()
            numberFormatter.numberStyle = NumberFormatter.Style.decimal
            let formattedNumber = numberFormatter.string(from: NSNumber(value:weekCount))
            
            attributedString = NSMutableAttributedString(string: formattedNumber!)
            attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-Bold", size: CGFloat(bigFontSize)) as Any, range: NSRange(location: 0, length: attributedString.length))
            attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(8.0), range: NSRange(location: 0, length: attributedString.length))
            weaksLivedValue.attributedText = attributedString
            
            growingAge = Int(round(progress * startAge))
            attributedString = NSMutableAttributedString(string: growingAge.description)
            attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
            attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-LightItalic", size: 16.0) as Any, range: NSRange(location: 0, length: attributedString.length))
            startAgeLabel.attributedText = attributedString
        }
        
        if isInPhase2 {
            
            yearsLeftArc.alpha = progress
            endAgeLabel.alpha = progress

            var currentWeeksRounded = Int(round(progress * CGFloat(numberOfWeeksLeft)))
            growingAge = Int(round(progress * CGFloat(endAge)))
        
            if (progress == 0.0) {
                currentWeeksRounded = numberOfWeeksLeft
                growingAge = Int(endAge)
                yearsLeftArc.alpha = 1.0
                endAgeLabel.alpha = 1.0
            }
            
            let numberFormatter = NumberFormatter()
            numberFormatter.numberStyle = NumberFormatter.Style.decimal
            let formattedNumber = numberFormatter.string(from: NSNumber(value:currentWeeksRounded))
        
            // Weeks left incrementing label
            attributedString = NSMutableAttributedString(string: formattedNumber!)
            attributedString.addAttribute(NSAttributedStringKey.font, value: UIFont(name: "Graphik-Bold", size: CGFloat(bigFontSize)) as Any, range: NSRange(location: 0, length: attributedString.length))
            attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(8.0), range: NSRange(location: 0, length: attributedString.length))
            weeksLeftValue.attributedText = attributedString
        
            // End age incrementing label
            attributedString = NSMutableAttributedString(string: growingAge.description)
            attributedString.addAttribute(NSAttributedStringKey.kern, value: CGFloat(3.0), range: NSRange(location: 0, length: attributedString.length))
            endAgeLabel.attributedText = attributedString
        }
    }
    
    func animationDidStart(_ anim: CAAnimation) {
        if(anim == yearsLivedArc.arc.animation(forKey:"strokeEnd")) {
            
            UIViewPropertyAnimator(duration: 0.5, curve: .easeOut, animations: {
                self.weaksLivedValue.alpha = 1
            }).startAnimation()
            
            rotatingArc1.startAnimation()
            rotatingArc2.startAnimation()
            rotatingArc3.startAnimation()
        }
    }
    
    func animationDidStop(_ anim: CAAnimation, finished flag: Bool) {
        
        let beginTime = CACurrentMediaTime() + Double(arc4random_uniform(24))
        let duration = Double(2.0 + Double((arc4random_uniform(16))))
        
        if(anim == yearsLivedArc.arc.animation(forKey:"strokeEnd")) {
            
            isInPhase1 = false
            
           //rotatingArc2.startAnimation()
            
            // Fade out years lived value and start new arc animation
            UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 2.5, delay: 0.5, options: [.curveEaseOut], animations: {
                self.weaksLivedValue.alpha = 0
            }, completion: nil)
            
            Timer.scheduledTimer(withTimeInterval: 0.75, repeats: false) { (timer) in
                self.startAnimPhase2()
                //self.rotatingArc3.startAnimation()
            }
            
            // Then fade out years lived label
            UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 1.75, delay: 1.0, options: [.curveEaseOut], animations: {
                self.weaksLivedLabel.alpha = 0
            }, completion: nil)
            
            // Then fade in years left value
            UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 3.0, delay: 1.5, options: [.curveEaseOut], animations: {
                self.weeksLeftValue.alpha = 1
            }, completion: nil)
            
            // Then fade in years left label
            UIViewPropertyAnimator.runningPropertyAnimator(withDuration: 2.5, delay: 1.6, options: [.curveEaseOut], animations: {
                self.weeksLeftLabel.alpha = 1
            }, completion: nil)
            
        }
        
        if(anim == rotatingArc1.layer.animation(forKey:"fadeIn")) {
            arc1FadeOut.beginTime = beginTime
            arc1FadeOut.duration = duration
            rotatingArc1.layer.add(arc1FadeOut, forKey: "fadeOut")
        }
        
        if(anim == rotatingArc1.layer.animation(forKey:"fadeOut")) {
            arc1FadeIn.beginTime = beginTime
            arc1FadeIn.duration = duration
            rotatingArc1.layer.add(arc1FadeIn, forKey: "fadeIn")
        }

        if(anim == rotatingArc2.layer.animation(forKey:"fadeIn")) {
            arc2FadeOut.beginTime = beginTime
            arc2FadeOut.duration = duration
            rotatingArc2.layer.add(arc2FadeOut, forKey: "fadeOut")
        }
        
        if(anim == rotatingArc2.layer.animation(forKey:"fadeOut")) {
            arc2FadeIn.beginTime = beginTime
            arc2FadeIn.duration = duration
            rotatingArc2.layer.add(arc2FadeIn, forKey: "fadeIn")
        }
        
        if(anim == rotatingArc3.layer.animation(forKey:"fadeIn")) {
            arc3FadeOut.beginTime = beginTime
            arc3FadeOut.duration = duration
            rotatingArc3.layer.add(arc3FadeOut, forKey: "fadeOut")
        }
        
        if(anim == rotatingArc3.layer.animation(forKey:"fadeOut")) {
            arc3FadeIn.beginTime = beginTime
            arc3FadeIn.duration = duration
            rotatingArc3.layer.add(arc3FadeIn, forKey: "fadeIn")
        }

    }
    
    @IBAction func onStartOverButtonTapped(_ sender: Any) {
        
        let viewControllers: [UIViewController] = self.navigationController!.viewControllers as [UIViewController];
        
        for aViewController:UIViewController in viewControllers {
            if aViewController.isKind(of: StartViewController.self) {
                _ = self.navigationController?.popToViewController(aViewController, animated: true)
            }
        }
    }
    
    func startAnimPhase2() {
        isInPhase2 = true
        yearsLeftArc.arc.add(ring2Animation, forKey: "ring2Animation")
        progressLayer2.add(animPhase2, forKey: "progress")
    }
    
    func showIconPicker() {
        let vc = self.storyboard?.instantiateViewController(withIdentifier: "IconChooserViewController") as! IconChooserViewController
        present(vc, animated: true, completion: nil)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        return .lightContent
    }
    
    override func viewWillAppear(_ animated: Bool) {
        prepViews()
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        restartAnimation()
    }

//    override var prefersStatusBarHidden: Bool {
//        return true
//    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
