//
//  GradientArcView.swift
//  HowManyWeeksLeft
//
//  Created by Justin Rhoades on 11.08.17.
//  Copyright © 2017 Justin Rhoades. All rights reserved.
//

import UIKit

@IBDesignable

class RotatingArcView: UIView, CAAnimationDelegate {
    
    var arcLayerA:CAShapeLayer! = nil
    var arcLayerB:CAShapeLayer! = nil
    
    @IBInspectable var offsetAmount: CGFloat = 13.0
    @IBInspectable var rotationDirection: CGFloat = 1.0
    @IBInspectable var rotationDuration: CGFloat = 10.0
    
    @IBInspectable var animationADuration: CGFloat = 2.0
    @IBInspectable var animationBDuration: CGFloat = 2.0

    @IBInspectable var arcAColor: UIColor = UIColor.darkGray
    @IBInspectable var arcALineWidth: CGFloat = 7.0
    @IBInspectable var strokeAStart: CGFloat = 0.0
    @IBInspectable var strokeAEnd: CGFloat = 1.0
    
    @IBInspectable var arcBColor: UIColor = UIColor.white
    @IBInspectable var arcBLineWidth: CGFloat = 7.0
    @IBInspectable var strokeBStart: CGFloat = 0.0
    @IBInspectable var strokeBEnd: CGFloat = 1.0

    var animationA = CAAnimationGroup()
    var rotationAnimationA = CABasicAnimation(keyPath:"transform.rotation")
    var strokeOnA = CABasicAnimation(keyPath:"strokeEnd")
    var strokeOffA = CABasicAnimation(keyPath:"strokeStart")
    var fadeInA = CABasicAnimation(keyPath:"opacity")
    var fadeOutA = CABasicAnimation(keyPath:"opacity")
    
    var animationB = CAAnimationGroup()
    var rotationAnimationB = CABasicAnimation(keyPath:"transform.rotation")
    var strokeOnB = CABasicAnimation(keyPath:"strokeEnd")
    var strokeOffB = CABasicAnimation(keyPath:"strokeStart")
    var fadeInB = CABasicAnimation(keyPath:"opacity")
    var fadeOutB = CABasicAnimation(keyPath:"opacity")
    
    var layerRotation = CABasicAnimation(keyPath:"transform.rotation")
    
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        if arcLayerA != nil { return }
        let path = UIBezierPath(ovalIn:self.bounds.insetBy(dx: offsetAmount, dy: offsetAmount))
        arcLayerA = CAShapeLayer()
        arcLayerA.frame = self.bounds
        arcLayerA.path = path.cgPath
        arcLayerA.fillColor = UIColor.clear.cgColor
        arcLayerA.strokeColor = arcAColor.cgColor
        arcLayerA.strokeStart = strokeAStart
        arcLayerA.strokeEnd = strokeAEnd
        arcLayerA.lineWidth = arcALineWidth
        arcLayerA.lineCap = kCALineCapRound
        arcLayerA.opacity = 0
        
        if arcLayerB != nil { return }
        //path = UIBezierPath(ovalIn:self.bounds.insetBy(dx: arcBLineWidth*3, dy: arcBLineWidth*3))
        //path = UIBezierPath(ovalIn:self.bounds.insetBy(dx: arcALineWidth*1.5, dy: arcALineWidth*1.5))
        arcLayerB = CAShapeLayer()
        arcLayerB.frame = self.bounds
        arcLayerB.path = path.cgPath
        arcLayerB.fillColor = UIColor.clear.cgColor
        arcLayerB.strokeColor = arcBColor.cgColor
        arcLayerB.strokeStart = strokeBStart
        arcLayerB.strokeEnd = strokeBEnd
        arcLayerB.lineWidth = arcBLineWidth
        arcLayerB.lineCap = kCALineCapRound
        arcLayerB.opacity = 0

        // Ring A Animations
        rotationAnimationA.fromValue = 0
        rotationAnimationA.toValue = .pi * 2.0
        rotationAnimationA.duration = Double(animationADuration)
        rotationAnimationA.isCumulative = true
        rotationAnimationA.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionLinear)
        rotationAnimationA.fillMode = kCAFillModeForwards
        rotationAnimationA.isRemovedOnCompletion = false
        
        fadeInA.duration = Double(animationADuration * 0.25)
        fadeInA.beginTime = 0.0
        fadeInA.fromValue = 0.0
        fadeInA.toValue = 1.0
        fadeInA.fillMode = kCAFillModeForwards
        fadeInA.isRemovedOnCompletion = false
        fadeInA.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        fadeOutA.duration = Double(animationADuration * 0.25)
        fadeOutA.beginTime = Double(animationADuration * 0.5)
        fadeOutA.fromValue = 1.0
        fadeOutA.toValue = 0.0
        fadeOutA.fillMode = kCAFillModeForwards
        fadeOutA.isRemovedOnCompletion = false
        fadeOutA.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        strokeOnA.duration = Double(animationADuration * 0.75)
        strokeOnA.beginTime = 0.0
        strokeOnA.isRemovedOnCompletion = false
        strokeOnA.fromValue = 0.0
        strokeOnA.toValue = 1.0
        strokeOnA.fillMode = kCAFillModeForwards
        strokeOnA.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        strokeOffA.duration = Double(animationADuration * 0.75)
        strokeOffA.beginTime = Double(animationADuration * 0.25)
        strokeOnA.isRemovedOnCompletion = false
        strokeOffA.fromValue = 0.0
        strokeOffA.toValue = 1.0
        strokeOffA.fillMode = kCAFillModeForwards
        strokeOffA.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)

        animationA.delegate = self;
        animationA.isRemovedOnCompletion = false
        animationA.fillMode = kCAFillModeForwards
        animationA.duration = Double(animationADuration)
      
        // Ring B Animations
        rotationAnimationB.fromValue = 0
        rotationAnimationB.toValue = .pi * 2.0
        rotationAnimationB.duration = Double(animationBDuration)
        rotationAnimationB.isCumulative = true
        rotationAnimationB.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionLinear)
        rotationAnimationB.fillMode = kCAFillModeForwards
        rotationAnimationB.isRemovedOnCompletion = false
        
        fadeInB.duration = Double(animationBDuration * 0.25)
        fadeInB.beginTime = 0.0
        fadeInB.fromValue = 1.0
        fadeInB.toValue = 0.0
        fadeInB.fillMode = kCAFillModeForwards
        fadeInB.isRemovedOnCompletion = false
        fadeInB.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        fadeOutB.duration = Double(animationBDuration * 0.25)
        fadeOutB.beginTime = Double(animationBDuration * 0.5)
        fadeOutB.fromValue = 0.0
        fadeOutB.toValue = 1.0
        fadeOutB.fillMode = kCAFillModeForwards
        fadeOutB.isRemovedOnCompletion = false
        fadeOutB.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        strokeOnB.duration = Double(animationBDuration * 0.75)
        strokeOnB.beginTime = 0.0
        strokeOnB.isRemovedOnCompletion = false
        strokeOnB.fromValue = 0.0
        strokeOnB.toValue = 1.0
        strokeOnB.fillMode = kCAFillModeForwards
        strokeOnB.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        strokeOffB.duration = Double(animationBDuration * 0.75)
        strokeOffB.beginTime = Double(animationBDuration * 0.25)
        strokeOnB.isRemovedOnCompletion = false
        strokeOffB.fromValue = 0.0
        strokeOffB.toValue = 1.0
        strokeOffB.fillMode = kCAFillModeForwards
        strokeOffB.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionEaseInEaseOut)
        
        animationB.delegate = self;
        animationB.isRemovedOnCompletion = false
        animationB.fillMode = kCAFillModeForwards
        animationB.duration = Double(animationBDuration)
        
        // Layer Animations
        layerRotation.fromValue = 0
        layerRotation.toValue = .pi * 2.0
        layerRotation.duration = Double(rotationDuration)
        layerRotation.isCumulative = true
        layerRotation.timingFunction = CAMediaTimingFunction(name:kCAMediaTimingFunctionLinear)
        layerRotation.delegate = self
        layerRotation.fillMode = kCAFillModeForwards
        layerRotation.isRemovedOnCompletion = false
        layerRotation.repeatCount = .greatestFiniteMagnitude
        
        // Add arc
        layer.addSublayer(arcLayerB)
        layer.addSublayer(arcLayerA)

        if rotationDirection == -1 {
            layer.transform = CATransform3DScale(CATransform3DMakeRotation(.pi / 2.0, 0, 0, 1), -1, 1, 1)
        }
    }
    
    func animationDidStop(_ anim: CAAnimation, finished flag: Bool) {
        
        if(anim == arcLayerA.animation(forKey:"animationA")) {
            arcLayerA.add(animationA, forKey: "animationA")
        }
        
        if(anim == arcLayerB.animation(forKey:"animationB")) {
            arcLayerB.add(animationB, forKey: "animationB")
        }
    }
    
    func startAnimation() {
        
        // Add Arc A Animations
        animationA.animations = [fadeInA, fadeOutA, rotationAnimationA, strokeOnA, strokeOffA]
        arcLayerA.add(animationA, forKey: "animationA")
        
        // Add Arc B Animations
        animationB.animations = [fadeInB, fadeOutB, rotationAnimationB, strokeOnB, strokeOffB]
        arcLayerB.add(animationB, forKey: "animationB")
        
        // Add Layer Animations
        layer.add(layerRotation, forKey: "layerRotation")
        
    }
    

    /*
    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    override func draw(_ rect: CGRect) {
        // Drawing code
    }
    */

}
